Type

Enum Type 

#[repr(u8)]
pub enum Type { User = 0, Kernel = 1, LowKernel = 2, GuestPhysical = 3, }

Variants



User = 0



Kernel = 1



LowKernel = 2

You probably do not want to use LowKernel. It is primarily used for SMP bootstrap or mexec to allow mappings of very low memory using the standard VMM subsystem.



GuestPhysical = 3

Used to construct an address space representing hypervisor guest memory.