VmMapping

Struct VmMapping 

pub struct VmMapping { /* private fields */ }
A leaf mapping that maps a VMO into the address space.

This is an opaque FFI wrapper around Zircon’s C++ VmMapping class.

Implementations

Source

impl VmMapping

pub fn destroy(&self) -> Result<(), Status>

Destroys this mapping, unmapping all pages and removing dependencies on the underlying VMO.

pub fn base(&self) -> usize

Returns the base virtual address of this mapping.

pub fn size(&self) -> usize

Returns the size in bytes of this mapping.

pub fn flags(&self) -> u32

Returns the creation flags of this mapping.

pub fn object_offset(&self) -> u64

Returns the offset into the underlying VMO for this mapping.

pub fn decommit_range(&self, offset: usize, len: usize) -> Result<(), Status>

Convenience wrapper for vmo()->DecommitRange() with the necessary offset modification and locking.

pub fn map_range( &self, offset: usize, len: usize, commit: bool, ignore_existing: bool, ) -> Result<(), Status>

Map in pages from the underlying vm object, optionally committing pages as it goes. |ignore_existing| controls whether existing hardware mappings in the specified range should be ignored or treated as an error. |ignore_existing| should only be set to true for user mappings where populating mappings may already be racy with multiple threads, and where we are already tolerant of mappings being arbitrarily created and destroyed.

pub fn debug_unmap(&self, base: usize, size: usize) -> Result<(), Status>

Unlocked convenience wrapper around unmap for testing.

pub fn debug_protect( &self, base: usize, size: usize, new_arch_mmu_flags: ArchMmuFlags, ) -> Result<(), Status>

Unlocked convenience wrapper around protect for testing.

pub fn vmo(&self) -> Option<RefPtr<VmObject>>

Returns the underlying VMO backing this mapping.