pub struct VmObject { /* private fields */ }Expand description
The base vm object that holds a range of bytes of data
Can be created without mapping and used as a container of data, or mappable into an address space via VmAddressRegion::CreateVmMapping