#[repr(u8)]pub enum NonTerminalAction {
FreeUnaccessed = 0,
Retain = 1,
}Variants
FreeUnaccessed = 0
If a non-terminal entry has no accessed information, unmap and free it. If it has accessed information, just remove the flag.
Retain = 1
Retain both the non-terminal mappings and any accessed information.