#[repr(u8)]pub enum TerminalAction {
UpdateAgeAndHarvest = 0,
UpdateAge = 1,
}Variants
UpdateAgeAndHarvest = 0
If the page is accessed update its age in the page queues, and remove the accessed flag.
UpdateAge = 1
If the page is accessed update its age in the page queues, but do not clear the flag.